package kerho;
import java.io.*;
import java.util.*;
import fi.jyu.mit.ohj2.*;
/**
 * Kerhon jsenist joka osaa mm. list uuden jsenen
 *
 * @author Vesa Lappalainen
 * @version 1.0, 22.02.2003
 * @version 1.1, 20.03.2003
 */
public class Jasenet {
  private static final int MAX_JASENIA = 25;
  private int lkm = 0;
  private boolean muutettu = false;
  private String nimi = "";
  private String koko_nimi = "";
  private Jasen alkiot[] = new Jasen[MAX_JASENIA];

  public Jasenet() {  }

  /**
   * Lis uuden jsenen tietorakenteeseen.  Ottaa jsenen omistukseensa.
   * @param jasen listvn jsenen viite.  Huom tietorakenne muuttuu omistajaksi
   * @throws SailoException jos tietorakennen on jo tynn
   */
  public void lisaa(Jasen jasen) throws SailoException {
    muutettu = true;
    if ( lkm >= alkiot.length ) throw new SailoException("Liikaa alkioita");
    alkiot[lkm] = jasen;
    lkm++;
  }

  /**
   * Korvaa jsenen tietorakenteessa.  Ottaa jsenen omistukseensa.
   * Etsitn samalla tunnusnumerolla oleva jsen.  Jos ei lydy,
   * niin listn uutena jsenen.
   * @param jasen listvn jsenen viite.  Huom tietorakenne muuttuu omistajaksi
   * @throws SailoException jos tietorakennen on jo tynn
   */
  public void korvaa(Jasen jasen) throws SailoException {
    int id = jasen.getTunnus_nro();
    for (int i = 0; i < lkm; i++ ) {
      if ( alkiot[i].getTunnus_nro() == id ) {
        alkiot[i] = jasen;
        muutettu = true;
        return;
      }
    }
    lisaa(jasen);
  }




  /**
   * Palauttaa viitteen i:teen jseneen.
   * @param i monennenko jsenen viite halutaan
   * @return viite jseneen, jonka indeksi on i
   * @throws SailoException jos i ei ole sallitulla alueella
   */
  public Jasen anna(int i) throws SailoException {
    if ( i < 0 || lkm <= i ) throw new SailoException("Laiton indeksi " + i);
    return alkiot[i];
  }





  /**
   * Lukee jsenistn tiedostosta.  Kesken.
   * @param tied tiedoston nimen alkuosa
   * @throws SailoException jos lukeminen eponnistuu
   */
  public void lue_tiedostosta(String tied) throws SailoException {
    nimi = tied;
    BufferedReader fi = Tiedosto.avaa_lukemista_varten(getTiedoston_nimi());
    if ( fi == null ) throw new SailoException("Tiedosto " + getTiedoston_nimi() + " ei aukea");

    try {
      koko_nimi = fi.readLine();
      if ( koko_nimi == null ) throw new SailoException("xxxxx");
      String rivi = fi.readLine();
      if ( rivi == null ) throw new SailoException("xxxxx");
      int max_koko = Mjonot.erotaInt(rivi,10); // tehdn jotakin

      while ( ( rivi = fi.readLine() ) != null ) {
        rivi = rivi.trim();
        if ( rivi.equals("") || rivi.startsWith(";") ) continue;
        Jasen jasen = new Jasen();
        jasen.parse(rivi);  // voisi olla virheksittely
        lisaa(jasen);
      }
      muutettu = false;


    } catch ( IOException e ) {


    } finally {
      try {
        fi.close();
      } catch (IOException ex) {
      }
    }
  }

  /**
   * Tallentaa jsenistn tiedostoon.  Kesken.
   * Tiedoston muoto:
   * <pre>
   * Kelmien kerho
   * 20
   * ; kommenttirivi
   * 2|Ankka Aku|121103-706Y|Ankkakuja 6|12345|ANKKALINNA|12-1234|||1996|50.0|30.0|Velkaa Roopelle
   * 3|Ankka Tupu|121153-706Y|Ankkakuja 6|12345|ANKKALINNA|12-1234|||1996|50.0|30.0|Velkaa Roopelle
   * </pre>
   * @throws SailoException jos talletus eponnistuu
   */
  public void talleta() throws SailoException {
    if ( !muutettu ) return;

    File fbak = new File(getBak_nimi());
    File ftied = new File(getTiedoston_nimi());
    if ( !fbak.delete() )         ; //System.err.println("Ei voi tuhota");
    if ( !ftied.renameTo(fbak) )  ; //System.err.println("Ei voi nimet");

    PrintWriter fo = Tiedosto.avaa_kirjoittamista_varten(ftied.getPath());
    if ( fo == null )
      throw new SailoException("Tiedosto " + ftied.getName() + "ei aukea");
    try {
      fo.println(getKoko_nimi());
      fo.println(alkiot.length);
      for (Iterator i=iterator(); i.hasNext(); ) {
        Jasen jasen = (Jasen)i.next();
        fo.println(jasen.toString());
      }
//    } catch ( IOException e ) {
    } finally {
      fo.close();
    }

    muutettu = false;
  }

  /**
   * Palauttaa Kerhon koko nimen
   * @return Kerhon koko nimi merkkijononna
   */
  public String getKoko_nimi()           { return koko_nimi;                   }

  /**
   * Palauttaa kerhon jsenten lukumrn
   * @return jsenten lukumr
   */
  public int getLkm()                    { return lkm;                         }

  /**
   * Palauttaa tiedoston nimen, jota kytetn tallennukseen
   * @return tallennustiedoston nimi
   */
  public String getTiedoston_nimi()      { return nimi + ".dat";               }

  /**
   * Palauttaa varakopiotiedoston nimen
   * @return varakopiotiedoston nimi
   */
  public String getBak_nimi()            { return nimi + ".bak";               }

  /**
   * Iteraattorin implementointi.
   * @author Vesa Lappalainen
   * @version 1.0, 20.03.2003
   */
  public class JasenetIterator implements Iterator {
    private int kohdalla = -1;

    public boolean hasNext() {
//     if ( kohdalla + 1 >= lkm )  return false;
//     return true;
      return kohdalla + 1 < lkm;
    }

    public Object next() throws NoSuchElementException {
      if ( !hasNext() ) throw new NoSuchElementException("Ei oo");
      kohdalla++;
      return alkiot[kohdalla];
    }

    public void remove() throws UnsupportedOperationException {
      throw new UnsupportedOperationException("Me ei poisteta");
    }
  }

  /**
   * Palauttaa iteraattorin jolla kaikki jsenet voidaan kyd lvitse
   * @return iteraattori kaikkien lpikymiseen
   */
  public Iterator iterator() {
    return new JasenetIterator();
  }

  public class Vertailija implements Comparator {
    int k;
    public Vertailija(int k) { this.k = k; }

    public int compare(Object o1, Object o2) {
      Jasen j1 = (Jasen)o1;
      Jasen j2 = (Jasen)o2;

      String s1 = j1.getAvain(k);
      String s2 = j2.getAvain(k);

      return s1.compareTo(s2);

    }

  }

  public List etsi(int k, String hakuehto) {
    List loytyneet = new ArrayList();

    for (Iterator i = iterator(); i.hasNext(); ) {
      Jasen jasen = (Jasen)i.next();
      String kentta = jasen.anna(k);
      if ( WildChars.onkoSamat(kentta,hakuehto) )
        loytyneet.add(jasen);
    }

    Collections.sort(loytyneet,new Vertailija(k));
    return loytyneet;
  }

  /**
   * Etsii jsenen id:n perusteella
   * @param id tunnusnumero, jonka mukaan etsitn
   * @return jsen jolla etsittv id tai null
   */
  public Jasen annaId(int id) {
    for (Iterator i = iterator(); i.hasNext(); ) {
      Jasen jasen = (Jasen)i.next();
      if ( id == jasen.getTunnus_nro() ) return jasen;
    }
    return null;

  }


  /**
   * Testiohjelma jsenistlle
   * @param args ei kytss
   */
  public static void main(String args[]) {
    Jasenet jasenet = new Jasenet();
    try {
      jasenet.lue_tiedostosta("kelmit");
    } catch ( SailoException ex ) {
      System.out.println("Virhe: " + ex.getMessage());
    }

    Jasen aku = new Jasen(), aku2 = new Jasen();
    aku.rekisteroi();    aku.vastaa_aku_ankka();
    aku2.rekisteroi();   aku2.vastaa_aku_ankka();

    try {
      jasenet.lisaa(aku);
      jasenet.lisaa(aku2);

      System.out.println("============= Jsenet testi =================");

//      for (int i=0; i<jasenet.getLkm(); i++) {
      for (Iterator  i=jasenet.iterator(); i.hasNext(); ) {
//        Jasen jasen = jasenet.anna(i);
        Jasen jasen = (Jasen)i.next();
        System.out.println("Jsen nro: " + jasen.getTunnus_nro());
        jasen.tulosta(System.out);
      }

    } catch ( SailoException ex ) {
      System.out.println(ex.getMessage());
    }

    try {
      jasenet.talleta();
    } catch ( SailoException e ) {
      System.out.println("Virhe: " + e.getMessage());
    }

  }

}


