import java.io.*;
import java.util.*;

import fi.jyu.mit.ohj2.*;
/**
 * Kyttliittymluokka Kerho-ohjelmaan
 * Kerho-ohjelman menujen testaus.  Varsinaiset toiminnot puuttuvat.<br>
 * Tarvitsee avukseen JAR-paketin:
 * <a href="http://www.mit.jyu.fi/vesal/kurssit/ohj2/ali/Ali.jar">Ali.jar</a><br>
 * Aliohjelmien dokumentaatio on:
 * <a href="http://www.mit.jyu.fi/vesal/kurssit/ohj2/ali/index.html">\kurssit\ohj2\ali</a><br>
 * <br>
 *
 * Listty tietorakenteen kytt jsenistn osalta.
 * Uuden jsenen lisminen lis aina vain Aku Ankan (tosi sat. hetulla).
 * Tulostus tulostaa kaikki jsenet.
 * Harrastusten osalta lis vain pitsin nyplyksi.
 *
 * Muutokset:  02.04.2003, tiedon sytt<br>
 *
 * @author  Vesa Lappalainen
 * @version 1.0, 09.02.2003
 * @version 1.1, 23.02.2003
 * @version 1.2, 20.03.2003
 * @version 1.3, 02.04.2003
 */
public class Naytto {

  /**
   * Viite kerhoon, jota tm kyttliittym kytt
   */
  private Kerho kerho;

  /**
   * Kyttliittymn alustus.  Kyttliittymlle PIT kerto mit
   * Kerho se ksittelee.
   * @param kerho jota ksitelln
   * @throws Exception jos kerhoa ei ole tuotu parametrina
   */
  public Naytto(Kerho kerho) throws Exception {
    if ( kerho == null ) throw new Exception("Kerho tytyy antaa!");
    this.kerho = kerho;
  }

  /**
   * Apualiohjelma tulostamiseen
   * @param s tuolsettava merkkijono
   * @param printlf tulostetaanko (true) rivinvaihto vai ei (false)
   */
  private static void tulosta(String s, boolean printlf) {
    System.out.print(s);
    if ( printlf ) System.out.println();
  }

  /**
   * Apualiohjelma tulostamiseen, tulostaa mys rivinvaihdon.
   * @param s tulostettava merkkijono
   */
  private static void tulosta(String s) {
    tulosta(s,true);
  }

  /**
   * Tulostaa n kappaletta tyhji rivej
   * @param n tulostettavien tyhjien rivien lukumr
   */
  private static void tyhja(int n) {
    for (int i=0; i<n; i++) tulosta("");
  }

  /**
   * Tulostaa kirjaimen ja tekstin alleviivattuna sek tyhj ymprille.
   * <pre>
   * Kutsu: otsikko('1',"Kissa istuu puussa");
   * tulostaa:
   * --- 4 tyhj rivi
   * 1. Kissa istuu puussa
   * =====================
   * --- 2 tyhj rivi
   * </pre>
   * @param kirjain alkuun tulostettava kirjain
   * @param teksti kirjaimen pern tulostettava. teksti
   */
  private static void otsikko(char kirjain, String teksti) {
    tyhja(4);
    tulosta("" + kirjain + ". " + teksti);
    tulosta(Mjonot.tayta("=",1+2+teksti.length()));
    tyhja(2);
  }

  /**
   * Tulostaa "Ei toimi viel" ja j odottamaan nppimen painallusta
   */
  void ei_toimi()  {
    tulosta("Ei toimi viel!");
    IO.odota_jotakin();
  }

  /**
   * Avustustoiminto
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  void avustus(char valinta) {
    otsikko(valinta, "Avustus");
    ei_toimi();
  }

  /**
   * Tulostaa yhden jsenen tiedot (mys harrastukset) valittuun
   * tietovirtaan.
   * @param os  tietovirta johon tulostetaan
   * @param jasen tulostettava jsen
   */
  private void tulosta(OutputStream os, final Jasen jasen) {
    jasen.tulosta(os);
    int jid = jasen.getTunnus_nro();
    for (Iterator i=kerho.harrastus_iterator(jid); i.hasNext(); ) {
      Harrastus har = (Harrastus)i.next();
      har.tulosta(os);
    }
  }

  /**
   * Kystn tiedot yhdelle jsenelle.  Muutetaan suoraan jsenen tietoja.
   * @param jasen muutettava jsen
   * @return halusiko kyttj pois muutoksesta (false) vai onnistuiko muutos (true)
   */
  private boolean kysy_tiedot(Jasen jasen) {

    int kenttia = jasen.getKenttia();
    int eka = jasen.ekaKentta();

    for (int k=eka; k<kenttia; k++ ) {
      while ( true ) {
        String oletus = jasen.anna(k);
        String kysymys = jasen.getKysymys(k);
        String vastaus = Syotto.kysy(kysymys,oletus);
        vastaus = Mjonot.poista_2_tyhjat(vastaus).trim();
        if ( k == eka && vastaus.equals("") ) return false;
        if ( vastaus.equals("q") ) return false;
        if ( vastaus.equals(".") ) return true;
        String virhe = jasen.aseta(k,vastaus);
        if ( virhe == null ) break;
        System.out.println("Virhe: " + virhe);
      }

    }
    return true;

/*
    tulosta("Toistaiseksi osaan list vain Aku Ankan");
    jasen.vastaa_aku_ankka();
    int lkm = Jasen.rand(0,5);
    for (int i=0; i<lkm; i++) {
      Harrastus har = new Harrastus();
      har.vastaa_pitsin_nyplays(jasen.getTunnus_nro());
      kerho.lisaa(har);
    }
*/
//    return true;
  }


  /**
   * Kysy tiedot uudelle jsenelle.  Luodaan uusi jsen ja rekisteridn.
   * Jos jsent ei halutakkaan list, niin mahdolliset hneen liittyvt
   * tiedot tulee poistaa.
   * @return halutaanko viel jatkaa (true) vai lopettaa kyseleminen (false)
   */
  private boolean kysy_uusi_jasen() {
    Jasen jasen = new Jasen();
    jasen.rekisteroi();
    while ( kysy_tiedot(jasen) ) {
      tulosta("Listnk");
      tulosta(System.out,jasen);
      tulosta(":",false);
      if ( IO.kylla_vastaus() ) {
        try {
          kerho.lisaa(jasen);
          return true;
        } catch ( SailoException ex ) {
          tulosta(ex.getMessage());
        }
      }
    }
    kerho.poista(jasen.getTunnus_nro());
    return false;
  }

  /**
   * Uuden jsenen lisys
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  void lisaa_uusi_jasen(char valinta) {
    otsikko(valinta,"Uuden jsenen lisys");
    while ( kysy_uusi_jasen() );
  }

  /**
   * Jsenen tietojen etsiminen
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  void etsi_jasenen_tiedot(char valinta)
  {
    otsikko(valinta,"Etsi jsenen tiedot");
    ei_toimi();
  }


  /**
   * Ohjelman tulosteet, pitisi tulla alimenu
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  void tulosteet(char valinta) {
    otsikko(valinta,"Tulosteet");
    tulosta("Toimii toistaiseksi vain nytlle");
    tulosta("---------------------------------------------------------");

    for (Iterator i=kerho.jasen_iterator(); i.hasNext(); ) {
      Jasen jasen = (Jasen)i.next();
      tulosta("Jsen nro: " + jasen.getTunnus_nro());
      tulosta(System.out,jasen);
      tulosta("");
    }

    tulosta("---------------------------------------------------------");
    IO.odota_jotakin();
  }


  /**
   * Tietojen korjailu
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  void tietojen_korjailu(char valinta) {
    otsikko(valinta,"Tietojen korjailu");
    ei_toimi();
  }


  /**
   * Jsenmaksujen pivitys
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  void paivita_jasenmaksuja(char valinta) {
    otsikko(valinta,"Pivit jsenmaksuja");
    ei_toimi();
  }


  /**
   * Tulostaa ohjelman pmenun
   */
  public void paamenu() {
    tulosta("\n\n\n\n");
    tulosta("Jsenrekisteri");
    tulosta("==============");
    tulosta("");
    tulosta("Kerhossa " + kerho.getNimi()+ " on " +
             kerho.getJasenia() + " jsent.");
    tulosta("");
    tulosta("Valitse:");
    tulosta("   ?  = avustus");
    tulosta("   0  = lopetus");
    tulosta("   1  = lis uusi jsen");
    tulosta("   2  = etsi jsenen tiedot");
    tulosta("   3  = tulosteet");
    tulosta("   4  = tietojen korjailu");
    tulosta("   5  = pivit jsenmaksuja");
    tulosta("   :",false);
  }

  /**
   * Silmukka jossa odotetaan nppint ja suoritetaan vastaava toiminto.
   * 0:n painaminen lopettaa silmukan ja palaa kutsuvaan ohjelmaan.
   * @return palauttaa 0 jos kaikki meni hyvin, 1 jos tuli virhe
   */
  int paavalinta() {
    char nappain;

    while ( true ) {

      paamenu();

      nappain = IO.odota_nappain("?012345",IO.EI_OLETUSTA,IO.MERKKI_ISOKSI);

      switch (nappain) {
        case '?': avustus(nappain);                break;
        case '0': return 0;
        case '1': lisaa_uusi_jasen(nappain);       break;
        case '2': etsi_jasenen_tiedot(nappain);    break;
        case '3': tulosteet(nappain);              break;
        case '4': tietojen_korjailu(nappain);      break;
        case '5': paivita_jasenmaksuja(nappain);   break;
        default : tulosta("Nin ei voi kyd!");  return 1;
      }

    }
  }

  /**
   * Tulostaa ohjelman logon
   */
  void logo() {
    tulosta("");
    tulosta("      ###################################");
    tulosta("      #   J  S E N R E K I S T E R I   #");
    tulosta("      #     versio 9.95                 #");
    tulosta("      #     Hannu Hanhi                 #");
    tulosta("      ###################################");
  }


  /**
   * Tulostaa ohjelman alkuohjeet
   */
  void ohjeet() {
    logo();
    tulosta("");
    tulosta("Tll ohjelmalla yllpidetn kerhon jsenrekisteri.");
  }


  /**
   * Tulostaa loppukiitokset
   */
  void kiitokset() {
    tulosta("");
    tulosta("KIITOS KYTST JA TERVETULOA UUDELLEEN");
    tulosta("");
    tulosta("");
  }


  /**
   * Lukee kerhon tiedot levylt.  Kysyy ensin kyttjlt tiedoson nimen.
   * @return palauttaa tyhjn merkkijonon jos kaikki menee hyvin, muuten
   * virhetekstin
   */
  String lue_kerho() {
    tulosta("Anna kerhon nimi>");
    ei_toimi();
    try {
      kerho.lue_tiedostosta("kelmit");
    } catch ( SailoException ex ) {
      System.out.println(ex.getMessage());
      return ex.getMessage();
    }

    return "";
  }


  /**
   * Tallentaa kerhon tiedot levylle.
   * @return palauttaa tyhjn merkkijonon jos kaikki menee hyvin, muuten
   * virhetekstin
   */
  String talleta_kerho() {
    logo();

    tulosta("");

    tulosta("Talletus...");
    try {
      kerho.talleta();
    } catch ( SailoException ex ) {
      System.out.println(ex.getMessage());
      return ex.getMessage();
    }

    return "";
  }


  /**
   * Ohjelman pohjelma
   * @param args komentorivin parametrit, ei kytss viel
   */
  public static void main(String[] args) {
    Kerho kerho = new Kerho();

    try {
      Naytto naytto = new Naytto(kerho);

      naytto.ohjeet();

      naytto.lue_kerho();

      naytto.paavalinta();

      naytto.talleta_kerho();

      naytto.kiitokset();
    }
    catch (Exception ex) {
      System.out.println("VIRHE: " + ex.getMessage());
    }
  }

}

