import fi.jyu.mit.ohj2.IO;
/**
 * Kyttliittymluokka Kerho-ohjelmaan
 * Kerho-ohjelman menujen testaus.  Varsinaiset toiminnot puuttuvat.<br>
 * Tarvitsee avukseen JAR-paketin:
 * <a href="http://www.mit.jyu.fi/vesal/kurssit/ohj2/ali/Ali.jar">Ali.jar</a><br>
 * Aliohjelmien dokumentaatio on:
 * <a href="http://www.mit.jyu.fi/vesal/kurssit/ohj2/ali/index.html">\kurssit\ohj2\ali</a><br>
 * <br>
 *
 * @author  Vesa Lappalainen
 * @version 1.0, 09.02.2003
 */
public class Naytto {

  /**
   * Viite kerhoon, jota tm kyttliittym kytt
   */
  private Kerho kerho;

  /**
   * Kyttliittymn alustus.  Kyttliittymlle PIT kerto mit
   * Kerho se ksittelee.
   * @param kerho jota ksitelln
   * @throws Exception jos kerhoa ei ole tuotu parametrina
   */
  public Naytto(Kerho kerho) throws Exception {
    if ( kerho == null ) throw new Exception("Kerho tytyy antaa!");
    this.kerho = kerho;
  }

  /**
   * Apualiohjelma tulostamiseen
   * @param s tuolsettava merkkijono
   * @param printlf tulostetaanko (true) rivinvaihto vai ei (false)
   */
  private static void tulosta(String s, boolean printlf) {
    System.out.print(s);
    if ( printlf ) System.out.println();
  }

  /**
   * Apualiohjelma tulostamiseen
   * @param s
   */
  public static void tulosta(String s) {
    tulosta(s,true);
  }

  /**
   * Tulostaa "Ei toimi viel" ja j odottamaan nppimen painallusta
   */
  private void ei_toimi()  {
    tulosta("Ei toimi viel!");
    IO.odota_jotakin();
  }

  /**
   * Avustustoiminto
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  private void avustus(char valinta) {
    tulosta("\n\n\n");
    tulosta(valinta +  ".  Avustus");
    tulosta(          "===========");
    ei_toimi();
  }

  /**
   * Uuden jsenen lisys
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  private void lisaa_uusi_jasen(char valinta) {
    tulosta("\n\n\n");
    tulosta(valinta +  ".  Uuden jsenen lisys");
    tulosta(          "========================");
    tulosta("");
    ei_toimi();
  }


  /**
   * Jsenen tietojen etsiminen
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  private void etsi_jasenen_tiedot(char valinta)
  {
    tulosta("\n\n\n");
    tulosta(valinta +  ".  Etsi jsenen tiedot");
    tulosta(          "=======================");
    ei_toimi();
  }


  /**
   * Ohjelman tulosteet, pitisi tulla alimenu
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  private void tulosteet(char valinta) {

    /*
      0 = pvlaintaan
      1 = tarrat
      2 = puhlu
      3 ...

    */
    tulosta("\n\n\n");
    tulosta(valinta +  ".  Tulosteet");
    tulosta(          "=============");
    ei_toimi();
  }


  /**
   * Tietojen korjailu
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  private void tietojen_korjailu(char valinta) {
    tulosta("\n\n\n");
    tulosta(valinta +  ".  Tietojen korjailu");
    tulosta(          "=====================");
    ei_toimi();
  }


  /**
   * Jsenmaksujen pivitys
   * @param valinta kirjain jolla toiminto aktivoitiin
   */
  private void paivita_jasenmaksuja(char valinta) {
    tulosta("\n\n\n");
    tulosta(valinta +  ".  Pivit jsenmaksuja");
    tulosta(          "========================");
    ei_toimi();
  }


  /**
   * Tulostaa ohjelman pmenun
   */
  private void paamenu() {
    tulosta("\n\n\n\n");
    tulosta("Jsenrekisteri");
    tulosta("==============");
    tulosta("");
    tulosta("Kerhossa " + kerho.getNimi()+ " on " +
             kerho.getJasenia() + " jsent.");
    tulosta("");
    tulosta("Valitse:");
    tulosta("   ?  = Avustus");
    tulosta("   0  = lOpetus");
    tulosta("   1  = Lis uusi jsen");
    tulosta("   2  = Etsi jsenen tiedot");
    tulosta("   3  = Tulosteet");
    tulosta("   4  = tietojen Korjailu");
    tulosta("   5  = pivit Jsenmaksuja");
    tulosta("   :",false);
  }

  /**
   * Silmukka jossa odotetaan nppint ja suoritetaan vastaava toiminto.
   * 0:n painaminen lopettaa silmukan ja palaa kutsuvaan ohjelmaan.
   * @return palauttaa 0 jos kaikki meni hyvin, 1 jos tuli virhe
   */
  public int paavalinta() {
    char nappain;

    while ( true ) {

      paamenu();

      nappain = IO.odota_nappain("?012345",IO.EI_OLETUSTA,IO.MERKKI_ISOKSI);

      switch (nappain) {
        case '?': avustus(nappain);                break;
        case '0': return 0;
        case '1': lisaa_uusi_jasen(nappain);       break;
        case '2': etsi_jasenen_tiedot(nappain);    break;
        case '3': tulosteet(nappain);              break;
        case '4': tietojen_korjailu(nappain);      break;
        case '5': paivita_jasenmaksuja(nappain);   break;
        default : tulosta("Nin ei voi kyd!");  return 1;
      }

    }
  }

  /**
   * Tulostaa ohjelman logon
   */
  private void logo() {
    tulosta("");
    tulosta("      ###################################");
    tulosta("      #   J  S E N R E K I S T E R I   #");
    tulosta("      #     versio 9.95                 #");
    tulosta("      #     Hannu Hanhi                 #");
    tulosta("      ###################################");
  }


  /**
   * Tulostaa ohjelman alkuohjeet
   */
  public void ohjeet() {
    logo();
    tulosta("");
    tulosta("Tll ohjelmalla yllpidetn kerhon jsenrekisteri.");
  }


  /**
   * Tulostaa loppukiitokset
   */
  public void kiitokset() {
    tulosta("");
    tulosta("KIITOS KYTST JA TERVETULOA UUDELLEEN");
    tulosta("");
    tulosta("");
  }


  /**
   * Lukee kerhon tiedot levylt.  Kysyy ensin kyttjlt tiedoson nimen.
   * @return palauttaa tyhjn merkkijonon jos kaikki menee hyvin, muuten
   * virhetekstin
   */
  public String lue_kerho() {
    tulosta("Anna kerhon nimi>");
    ei_toimi();
    kerho.lue_tiedostosta("kelmit");

    return "";
  }


  /**
   * Tallentaa kerhon tiedot levylle.
   * @return palauttaa tyhjn merkkijonon jos kaikki menee hyvin, muuten
   * virhetekstin
   */
  public String talleta_kerho() {
    logo();

    tulosta("");

    tulosta("Talletus...");
    ei_toimi();

    return "";
  }


  /**
   * Ohjelman pohjelma
   * @param args komentorivin parametrit, ei kytss viel
   */
  public static void main(String[] args) {
    Kerho kerho = new Kerho();

    try {
      Naytto naytto = new Naytto(kerho);

      naytto.ohjeet();

      naytto.lue_kerho();

      naytto.paavalinta();

      naytto.talleta_kerho();

      naytto.kiitokset();
    }
    catch (Exception ex) {
      System.out.println("VIRHE: " + ex.getMessage());
    }
  }

}

