import java.io.*;
import java.util.*;
import fi.jyu.mit.ohj2.*;
/**
 * Kerhon jsenist joka osaa mm. list uuden jsenen
 *
 * @author Vesa Lappalainen
 * @version 1.0, 22.02.2003
 * @version 1.1, 20.03.2003
 */
public class Jasenet {
  private static final int MAX_JASENIA = 5;
  private int lkm = 0;
  private boolean muutettu = false;
  private String tiedoston_nimi = "";
  private String bak_nimi = "";
  private String koko_nimi = "";
  private Jasen alkiot[] = new Jasen[MAX_JASENIA];

  public Jasenet() {  }

  /**
   * Poikkeusluokka tietorakenteesta aiheutuville poikkeuksille.
   * @author Vesa Lappalainen
   * @version 1.0, 22.02.2003
   */
  public class SailoException extends Exception {
    public SailoException(String viesti) { super(viesti); }
  }

  /**
   * Lis uuden jsenen tietorakenteeseen.  Ottaa jsenen omistukseensa.
   * @param jasen listvn jsnen viite.  Huom tietorakenne muuttuu omistajaksi
   * @throws SailoException jos tietorakennen on jo tynn
   */
  public void lisaa(Jasen jasen) throws SailoException {
    if ( lkm >= alkiot.length ) throw new SailoException("Liikaa alkioita");
    alkiot[lkm] = jasen;
    lkm++;
  }


  /**
   * Palauttaa viitteen i:teen jseneen.
   * @param i monennenko jsenen viite halutaan
   * @return viite jseneen, jonka indeksi on i
   * @throws SailoException jos i ei ole sallitulla alueella
   */
  public Jasen anna(int i) throws SailoException {
    if ( i < 0 || lkm <= i ) throw new SailoException("Laiton indeksi " + i);
    return alkiot[i];
  }





  /**
   * Lukee jsenistn tiedostosta.  Kesken.
   * @param tied tiedoston nimen alkuosa
   * @throws SailoException jos lukeminen eponnistuu
   */
  public void lue_tiedostosta(String tied) throws SailoException {
    tiedoston_nimi = tied + ".dat";
    koko_nimi = "Kelmien kerho";
  }

  /**
   * Tallentaa jsenistn tiedostoon.  Kesken.
   * @throws SailoException jos talletus eponnistuu
   */
  public void talleta() { }

  /**
   * Palauttaa Kerhon koko nimen
   * @return Kerhon koko nimi merkkijononna
   */
  public String getKoko_nimi()           { return koko_nimi;                   }

  /**
   * Palauttaa kerhon jsenten lukumrn
   * @return jsenten lukumr
   */
  public int getLkm()                    { return lkm;                         }

  /**
   * Palauttaa tiedoston nimen, jota kytetn tallennukseen
   * @return tallennustiedoston nimi
   */
  public String getTiedoston_nimi()      { return tiedoston_nimi;              }

  /**
   * Palauttaa varakopiotiedoston nimen
   * @return varakopiotiedoston nimi
   */
  public String getBak_nimi()            { return bak_nimi;                    }

  /**
   * Tekee nykyisest tiedostosta varakopiotiedoston. Kesken.
   * @param bak_tark tarkennin varakopioiedostoille
   * @return onnistuiko (true) vai ei (false)
   */
  public boolean TeeBak(String bak_tark) { bak_nimi = bak_tark; return true;   }

  /**
   * Iteraattorin implementointi.
   * @author Vesa Lappalainen
   * @version 1.0, 20.03.2003
   */
  public class JasenetIterator implements Iterator {
    private int kohdalla = -1;

    public boolean hasNext() {
//     if ( kohdalla + 1 >= lkm )  return false;
//     return true;
      return kohdalla + 1 < lkm;
    }

    public Object next() throws NoSuchElementException {
      if ( !hasNext() ) throw new NoSuchElementException("Ei oo");
      kohdalla++;
      return alkiot[kohdalla];
    }

    public void remove() throws UnsupportedOperationException {
      throw new UnsupportedOperationException("Me ei poisteta");
    }
  }

  /**
   * Palauttaa iteraattorin jolla kaikki jsenet voidaan kyd lvitse
   * @return iteraattori kaikkien lpikymiseen
   */
  public Iterator iterator() {
    return new JasenetIterator();
  }



  /**
   * Testiohjelma jsenistlle
   * @param args ei kytss
   */
  public static void main(String args[]) {
    Jasenet jasenet = new Jasenet();

    Jasen aku = new Jasen(), aku2 = new Jasen();
    aku.rekisteroi();    aku.vastaa_aku_ankka();
    aku2.rekisteroi();   aku2.vastaa_aku_ankka();

    try {
      jasenet.lisaa(aku);
      jasenet.lisaa(aku2);

      System.out.println("============= Jsenet testi =================");

//      for (int i=0; i<jasenet.getLkm(); i++) {
      for (Iterator  i=jasenet.iterator(); i.hasNext(); ) {
//        Jasen jasen = jasenet.anna(i);
        Jasen jasen = (Jasen)i.next();
        System.out.println("Jsen nro: " + i);
        jasen.tulosta(System.out);
      }

    } catch ( Jasenet.SailoException ex ) {
      System.out.println(ex.getMessage());
    }
  }

}


